/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef PARCEL_INTERNAL_H_
#define PARCEL_INTERNAL_H_

#include <bundle.h>

#include <vector>
#include <iterator>
#include <algorithm>
#include <string>

namespace rpc_port {

class Parcel {
 public:
  Parcel() = default;
  Parcel(const unsigned char* buf, unsigned int size);

  void Write(const unsigned char* buf, unsigned int size);
  void Read(unsigned char* buf, unsigned int size);
  void WriteByte(char b);
  void WriteInt16(short i);
  void WriteInt32(int i);
  void WriteInt64(long long i);
  void WriteFloat(float f);
  void WriteDouble(double d);
  void WriteString(const char* str);
  void WriteBool(bool b);
  void WriteBundle(bundle* b);
  void WriteArrayCount(int count);
  char ReadByte();
  short ReadInt16();
  int ReadInt32();
  long long ReadInt64();
  float ReadFloat();
  double ReadDouble();
  const char* ReadString();
  bool ReadBool();
  bundle* ReadBundle();
  int ReadArrayCount();
  const std::vector<unsigned char>& GetRaw();

 private:
  template<typename T>
  void Write(T d) {
    unsigned char* p = reinterpret_cast<unsigned char*>(&d);

    std::copy(p, p + sizeof(T), std::back_inserter(data_));
  }

  template<typename T>
  T Read() {
    T d = 0;
    unsigned char* p = reinterpret_cast<unsigned char*>(&d);

    if (reader_ + sizeof(T) > data_.size())
      return d;
    std::copy(&data_[reader_], &data_[reader_] + sizeof(T), p);
    reader_ += sizeof(T);
    return d;
  }

 private:
  std::vector<unsigned char> data_;
  unsigned int reader_ = 0;
};

}  // namespace rpc_port

#endif  // PARCEL_INTERNAL_H_
