/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef PORT_INTERNAL_H_
#define PORT_INTERNAL_H_

#include <thread>
#include <mutex>
#include <string>
#include <memory>

namespace rpc_port {
namespace internal {

class Port {
 public:
  Port(int fd, std::string id, std::string instance);
  Port(int fd, std::string id);
  virtual ~Port();

  int Read(void* buf, unsigned int size);
  int Write(const void* buf, unsigned int size);
  int GetFd() const {
    return fd_;
  }

  const std::string& GetId() const {
    return id_;
  }

  std::recursive_mutex& GetMutex() const {
    return mutex_;
  }

  const std::string& GetInstance() const {
    return instance_;
  }

 private:
  int fd_;
  std::string id_;
  std::string instance_;
  mutable std::recursive_mutex mutex_;
};

}  // namespace internal
}  // namespace rpc_port

#endif  // PORT_INTERNAL_H_
