/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef STUB_INTERNAL_H_
#define STUB_INTERNAL_H_

#include <glib.h>
#include <gio/gio.h>
#include <glib-unix.h>

#include <string>
#include <list>
#include <memory>

#include "port-internal.h"
#include "fdbroker-internal.h"

namespace rpc_port {
namespace internal {

class Stub : private FdBroker::IEventListener {
 public:
  class IEventListener {
   public:
    virtual void OnConnected(const std::string& sender,
                             const std::string& instance) = 0;
    virtual void OnDisconnected(const std::string& sender,
                                const std::string& instance) = 0;
    virtual int OnReceived(const std::string& sender,
                           const std::string& instance, Port* port) = 0;
  };

  explicit Stub(const std::string& port_name, bool mock = false);
  virtual ~Stub();

  int Listen(IEventListener* ev);
  void AddPrivilege(const std::string& privilege);
  void SetTrusted(const bool trusted);
  std::shared_ptr<Port> FindPort(const std::string& instance) const;
  std::shared_ptr<Port> FindDelegatePort(const std::string& instance) const;

 private:
  class AcceptedPort : public Port {
   public:
    AcceptedPort(Stub* parent, bool isDelegate, int fd, std::string id,
        std::string inst, bool receive);
    AcceptedPort(Stub* parent, bool isDelegate, int fd, std::string id,
        bool receive);
    virtual ~AcceptedPort();
    bool IsDelegate() const {
      return is_delegate_;
    }

   private:
    int Watch(bool receive);

   private:
    GIOChannel* gioc_ = nullptr;
    int disconn_src_ = 0;
    int src_ = 0;
    Stub* parent_;
    bool is_delegate_ = false;
  };

  static gboolean OnDataReceived(GIOChannel *gio, GIOCondition cond,
                                 gpointer data);
  static gboolean OnSocketDisconnected(GIOChannel *gio, GIOCondition cond,
                                       gpointer data);

  void OnFdReceived(const std::string& sender, int fds[2]) override;
  void RemoveAcceptedPorts(std::string instance);

 private:
  std::list<std::shared_ptr<AcceptedPort>> ports_;
  IEventListener* listener_ = nullptr;
  FdBroker fd_broker_;
  std::string port_name_;
};

}  // namespace internal
}  // namespace rpc_port

#endif  // STUB_INTERNAL_H_
